#include <stdio.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "iir.h"
#include "vec.h"

#define myabs(x) (x>=0 ? x :-x)


extern int Log, ChunkSize;
extern char HangCmd[];

/*
 * Returns true if we have a ppp connection up and running
 */
int connected()
{
    FILE * f;
    char line[500];

    f= fopen( "/proc/net/dev", "r" );
    if ( f==NULL )
    {
        fprintf( stderr, "Error: No puedo abrir /proc/dev/net.\n" );
        exit(-1);
    }

    while ( !feof( f ) )
    {
        fgets( line, 500, f );
        if ( strstr(line, "ppp") )
        {
            fclose( f );
            return 1;
        }    
    }

    fclose( f );
    return 0;
}

/*
 * Reads buffLen bytes from fd and store them on buff  
 * Returns the number of signed short read 
 */
int readChunk( int fd, signed short * buff, long buffLen )
{
    int len;

    if ((len = read(fd, buff, buffLen)) == -1) 
    {
        perror("audio read");
        exit(1);
    }

    return len/sizeof(signed short);
}

/*
 * Handles state change 
 */
void stateDetect( int * state, unsigned long * pulses, float * totalAcum
    , float chunkMean, int nChunk, int chunkSize )
{
    float totalMean, limit;

    (*totalAcum)+= chunkMean;
    totalMean= (*totalAcum)/nChunk;
    limit= totalMean*3;

#ifdef DEBUG
    printf( "%lu: cMean=%f, tMean=%f, limit=%f\n"
        , (unsigned long)(((nChunk-1)*chunkSize*1000)/8000), chunkMean
        , totalMean, limit );
#endif                    


    switch( *state )
    {
        case 0:
            if ( chunkMean >= limit )
            {
#ifdef DEBUG
                printf("%lu - state 0 -> 1 - pulses %lu\n"
                    , (unsigned long)(((float)nChunk*chunkSize*1000)/8000)
                    , *pulses);
#endif                                
                *state= 1;
                *pulses= 1;
            }    
            break;
        case 1:
            if ( chunkMean >= limit )
                (*pulses)++;
            else
            {
                if (*pulses>=1 && *pulses<=3)
                {
#ifdef DEBUG
                    printf("%lu - state 1 -> 2 - pulses %lu\n"
                        , (unsigned long)(((float)nChunk*chunkSize*1000)/8000)
                        , *pulses);
#endif                                
                    *state= 2;
                }    
                else
                {
#ifdef DEBUG
                    printf("%lu - state 1 -> 0 - pulses %lu\n"
                        , (unsigned long)(((float)nChunk*chunkSize*1000)/8000)
                        , *pulses);
#endif                                
                    *state= 0;
                }
                *pulses= 1;
            }    
            break;
        case 2:
            if ( chunkMean >= limit )
            {
                if (*pulses>=1 && *pulses<=3)
                {
#ifdef DEBUG
                    printf("%lu - state 2 -> 3 - pulses %lu\n"
                        , (unsigned long)(((float)nChunk*chunkSize*1000)/8000)
                        , *pulses);
#endif                                
                    *state= 3;
                }    
                else
                {
#ifdef DEBUG
                    printf("%lu - state 2 -> 1 - pulses %lu\n"
                        , (unsigned long)(((float)nChunk*chunkSize*1000)/8000)
                        , *pulses);
#endif                                
                    *state= 1;
                }
                *pulses= 1;
            }    
            else
            {
                (*pulses)++;
            }    
            break;
        case 3:
            if ( chunkMean >= limit )
                (*pulses)++;
            else
            {
                if (*pulses>=1 && *pulses<=3)
                {
#ifdef DEBUG
                    printf("%lu - state 3 -> 4 - pulses %lu\n"
                        , (unsigned long)(((float)nChunk*chunkSize*1000)/8000)
                        , *pulses);
#endif                                
                    *state= 4;
                }    
                else
                {
#ifdef DEBUG
                    printf("%lu - state 3 -> 0 - pulses %lu\n"
                        , (unsigned long)(((float)nChunk*chunkSize*1000)/8000)
                        , *pulses);
#endif                                
                    *state= 0;
                }
                *pulses= 1;
            }    
            break;
        case 4:
#ifdef DEBUG
            printf("%lu - LLAMADA DETECTADA!\n"
                , (unsigned long)(((float)nChunk*chunkSize*1000)/8000));
#endif                            
            fprintf(stderr, "%lu - LLAMADA DETECTADA!\n"
                , (unsigned long)(((float)nChunk*chunkSize*1000)/8000));
            printf( "HANGCMD= %s\n", HangCmd );
            system( HangCmd );
    
            *state= 0;
            *pulses= 1;
    }
}


/*
 * It does the detection...
 * 
 * Read samples from fd, uses buff for storing and put filtered data on filtered
 * (both with buffLen size)
 * And uses the coefficients from a & b for the IIR filtering
 */
void detect( int fd, signed short * buff, float * filtered, long buffLen
    , float * a, float * b )
{
    int i, state= 0;
    float chunkAcum=0, chunkMean= 0, totalAcum= 0;
    long cont2= 0, readLen= buffLen;
    FILE * f2= NULL, * fMean= NULL;
    unsigned long pulses= 0, cont= 0, nChunk= 0;

    if ( Log )
    {
        f2= fopen( "f2-detect.dat", "w" );
        fMean= fopen( "f2-mean.dat", "w" );
    }    

    while ( 1 )
    {
        filtered[0]= filtered[readLen - 2];
        filtered[1]= filtered[readLen - 1];

        readLen= readChunk( fd, buff, buffLen );

        recursiveNarrow( buff, filtered, readLen, a, b );

        if ( Log )
            writeVecN( f2, filtered, readLen, cont );
    
        for ( i= 0; i<readLen; i++ )
        {
            if( (cont2) == ChunkSize )
            {
                if ( Log )
                {
                    fprintf( fMean, "%ld\t%f\n", nChunk, chunkMean );
                    fflush( fMean );
                }    
                cont2= 0;
                nChunk++;


                chunkMean= chunkAcum/ChunkSize;

                /* Every x chunks test if ppp connection is up */
                if ( (nChunk % 32) == 0 )
                {
                    if (! connected())
                        return;
                }

                stateDetect( &state, &pulses, &totalAcum, chunkMean, nChunk
                            , ChunkSize );
                chunkAcum= 0;            
            }

            chunkAcum+= myabs( filtered[i] ); 
            cont2++;
        }    
        cont+= readLen;
    }

    if ( Log )
    {
        fclose( f2 );
        fclose( fMean );
    }    
}
