#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <linux/soundcard.h>
#include "vec.h"
#include "iir.h"
#include "dsp.h"
#include "detect.h"
#include "config.h"

#define CONF "tuxcallrc"

#define FORMAT AFMT_S16_NE
#define CHANNELS 1
#define SPEED 8000

#define BUFFLEN 4096
#define CMDLEN 200
#define FNAMELEN 200

/* Globals */
int Log= 0 /* Log activated? */
  , ChunkSize= 640; /* Sample group's size */
float Freq= 420; /* Call waiting tone frequency */
char HangCmd[CMDLEN]= "killall -9 pppd"; /* Command for hanging connection */

void use()
{
    printf( "TuxCall - version %s\n\n", VERSION );
    printf( "uso: tuxcall [-l] [-f frecuencia] [-e comando]\n" );
    printf( "   -l              Guarda seal filtrada y medias de esta en ficheros\n                   f2-detect.dat y f2-mean.dat\n" );
    /*printf( "   -s valor        Todo lo que queda por debajo de \"valor\" se considera como\n                   ausencia de seal. (Por defecto %.1f)\n", Silence );*/
    printf( "   -f frecuencia   Frecuencia del tono de llamada en espera en herzios.\n                   (Por defecto %.0f)\n", Freq );
    printf( "   -e \"comando\"    Comando que se ejecuta para cortar la conexin\n" );
}

void allocateBuff( signed short ** buff, float ** filtered, long buffLen )
{
    *buff= malloc( buffLen * sizeof(signed short) );
    *filtered= malloc( buffLen * sizeof(float) );
}

void freeBuff( signed short * buff, float * filtered )
{
    free( buff );
    free( filtered );
}

/*
 * Parse program arguments
 */
void parseArgs( int argc, char ** argv )
{
    int i;

    for ( i=1; i<argc; i++ )
    {
        if ( strcmp(argv[1], "-l") == 0 )
        {
            Log= 1;
        }
        /*else if ( strcmp(argv[1], "-s") == 0 )
        {
            if ( i< argc-1 )
            {
                Silence= atof( argv[++i] );
            }
            else
            {
                use();
                exit(-1);
            }
        }*/
        else if ( strcmp(argv[1], "-f") == 0 )
        {
            if ( i< argc-1 )
            {
                Freq= atof( argv[++i] );
            }
            else
            {
                use();
                exit(-1);
            }
        }    
        else if ( strcmp(argv[1], "-e") == 0 )
        {
            if ( i< argc-1 )
            {
                strncpy( HangCmd, argv[++i], CMDLEN-1 );
                HangCmd[CMDLEN-1]= 0;
            }
            else
            {
                use();
                exit(-1);
            }
        }    
        else    
        {
            use();
            exit(-1);
        }
    }    
}

int main( int argc, char ** argv )
{
    float a[3], b[3];
    float center, bw;
    int dsp;
    signed short * buff;
    float * filtered;
    char confFile[FNAMELEN];
    FILE * f;

    bw= 0.00125;

    /* Loading global config */
    snprintf( confFile, FNAMELEN-1, "/etc/%s", CONF );
    f= fopen( confFile, "r" );
    if ( f != NULL )
    {
/*#ifdef DEBUG
        printf( "Cargando %s ...\n", confFile );
#endif*/
    	loadConfig( f );
        fclose( f );
    }

    /* Loading user config */
    snprintf( confFile, FNAMELEN-1, "%s/.%s", getenv("HOME"), CONF );
    f= fopen( confFile, "r" );
    if ( f != NULL )
    {
/*#ifdef DEBUG
        printf( "\nCargando %s ...\n", confFile );
#endif*/
    	loadConfig( f );
        fclose( f );
    }

    parseArgs( argc, argv );

    center= Freq/8000;

    allocateBuff( &buff, &filtered, BUFFLEN );

    iirPassband( a, b, center, bw );

#ifdef DEBUG    
    /*printf( "silence= %f\n", Silence );*/
    printf( "\ncomando= %s\n", HangCmd );
    printf( "frecuencia= %f\n", Freq );
    printf( "center= %f, bw= %f\n", center, bw );
    printf( "a0= %40.43f\na1= %40.43f\na2= %40.43f\nb1= %40.43f\nb2= %40.43f\n"
        , a[0], a[1], a[2], b[1], b[2] );
#endif         

    fprintf( stderr, "Esperando conexin ppp... ");
    fflush( stderr );

    while ( 1 )
    {
        if ( connected() )
        {
            fprintf( stderr, "detectada.\n" );

            printf( "Abriendo /dev/dsp... " );
            fflush( stdout );
            dsp= initDSP( FORMAT, CHANNELS, SPEED );
            if ( dsp == -1 )
            {
                fprintf( stderr, "Error abriendo /dev/dsp...\n" );
                exit( -1 );
            }
            printf( "hecho.\n" );
            fprintf( stderr, "Detectando...\n" );
            detect( dsp, buff, filtered, BUFFLEN, a, b );
            close( dsp );

            fprintf( stderr, "Esperando conexin ppp... ");
            fflush( stderr );
        }    
        else
            sleep( 5 );
    }    

    freeBuff( buff, filtered );

    return 0;
}


